'''
    PyBB - Logger Module

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''

from __future__ import generators
import os, sys
import time
import re
import config

ERROR_LOG_FILE = 'error.log'
LOG_FILE = 'user.log'
WRITE_LOG_FILE = 'userWrite.log'

#if os.environ.get('SERVER_SOFTWARE', '').startswith('Medusa'):
#    logCache = cache.Cache()
#else:
#    logCache = None

class ErrorLogger:
    def __init__(self, logging=True):
        self.logging = logging

    def log(self, fileName, errMsg):
        if self.logging:
            f = open(os.path.join(config.systemFolder, ERROR_LOG_FILE), 'a')
            f.write('[%s], [%s], %s, [%s], [%s], [%s]\n' % (
                        time.strftime('%Y/%m/%d:%H:%M:%S'),
                        fileName,
                        errMsg,
                        os.environ.get('REMOTE_ADDR', ''),
                        os.environ.get('REQUEST_URI', ''),
                        os.environ.get('HTTP_USER_AGENT', '')))

def log(uid, msg, arg=''):
    '''Log Information
    userId, ip, action, (action parameters), url, time, agent
    '''
    logMessage = '%s %s %s (%s) [%s] [%s] [%s]\n' % (
                uid,
                os.environ.get('REMOTE_ADDR', ''),
                msg,
                arg,
                os.environ.get('REQUEST_URI', ''),
                time.strftime('%Y/%m/%d:%H:%M:%S'),
                os.environ.get('HTTP_USER_AGENT', ''))
    #f = open('%s%s' % (LOG_FILE, time.strftime('%Y%m%d')), 'a')   # daily log
    f = open(os.path.join(config.systemFolder, LOG_FILE), 'a')
    f.write(logMessage)
    if msg in ('do_newThread', 'do_reply', 'do_deleteArticle', 'do_editArticle',
                    'do_deleteThread', 'do_moveThread'):
        f = open(os.path.join(config.systemFolder, WRITE_LOG_FILE), 'a')
        f.write(logMessage)
        f.close()
        packWriteLogFile(agoInSec=config.writeLog_TimeWindowSize)

def reverseLineGenerator(fname):
    f = open(os.path.join(config.systemFolder, fname))
    f.seek(0, 2)
    size = f.tell()
    L = []
    k = 1
    while k <= size:
        f.seek(-k, 2)
        ch = f.read(1)
        if ch == '\n':
            if L:
                L.reverse()
                yield ''.join(L)
                L = []
        else:
            L.append(ch)
        k += 1
    if L:
        L.reverse()
        yield ''.join(L)

class LogInfo:
    paramp = re.compile('\((.*?)\)')
    bracketp = re.compile('\[(.*?)\]')
    fromto = re.compile('from:(.*?) to:(.*)')
    groupMailp = re.compile('groups:(.*?) subject:(.*)')
    levelChangep = re.compile('(\S+) from:(\S+) to:(.*)')
    def __init__(self, line):
        L = line.split()
        self.uid, self.ip, self.action = L[:3]
        self.params = self.paramp.findall(line)[0]
        if self.action == 'do_reply':
            self.forumId, self.threadId, tmp, self.articleId = \
                self.params.split()
        elif self.action == 'do_editArticle':
            self.forumId, self.threadId, self.articleId = \
                self.params.split()
        elif self.action == 'do_deleteThread':
            self.forumId, self.threadId = \
                self.params.split()
        elif self.action == 'do_deleteArticle':
            self.forumId, self.threadId, self.articleId = \
                self.params.split()
        elif self.action == 'do_newThread':
            args = self.params.split()
            if len(args) == 2:
                self.forumId, self.threadId = args
            elif len(args) == 3:
                self.forumId, self.threadId, self.articleId = args
        elif self.action == 'do_showThread':
            self.forumId, self.threadId = \
                self.params.split()
        elif self.action in ('do_moveThread', 'do_copyThread'):
            m = self.fromto.match(self.params)
            self.fromForumId = m.group(1).split()[0]
            self.fromThreadIdList = m.group(1).split()[1:]
            self.toForumId = m.group(2).split()[0]
            self.toThreadIdList = m.group(2).split()[1:]
        elif self.action == 'do_groupMail':
            m = self.groupMailp.match(self.params)
            self.groups = m.group(1).split()
            self.subject = m.group(2)
        elif self.action == 'LevelChange':
            m = self.levelChangep.match(self.params)
            self.changedUid = m.group(1)
            self.fromLevel = int(m.group(2))
            self.toLevel = int(m.group(3))
        self.url, self.time, self.browser = self.bracketp.findall(line)[-3:]
    def __repr__(self):
        return '%s %s %s (%s) [%s] [%s] [%s]' % (self.uid, self.ip, self.action,
                                                 self.params, self.url, self.time, self.browser)

class WebLogAnalyzer:
    def __init__(self, fname=''):
        if fname:
            self.fname = fname
        else:
            self.fname = LOG_FILE
        if not os.path.exists(os.path.join(config.systemFolder, self.fname)):
            file(os.path.join(config.systemFolder, self.fname), 'w').write('')

    def logGenerator(self, actionList, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        for line in file(os.path.join(config.systemFolder, self.fname)):
            #uid, ip, action = line.split()[:3]
            try:
                log = LogInfo(line)
            except:
                print 'Failed to parse : %s' % line
                continue
            if actionList=='*' or log.action in actionList:
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                count += 1
                if N != 0 and count >= N:
                    break

    def recentLogGenerator(self, actionList, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        for line in reverseLineGenerator(self.fname):
            #uid, ip, action = line.split()[:3]
            try:
                log = LogInfo(line)
            except:
                print 'Failed to parse : %s' % line
                continue
            if log.action in actionList:
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                count += 1
                if N != 0 and count >= N:
                    break

    def getRecentLogsForReplies(self, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        for log in self.recentLogGenerator(('do_reply', 'do_deleteArticle')):
            if log.action in ('do_reply',):
                if (log.forumId, log.articleId) in deletedArticleList:
                    continue
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                count += 1
                if N != 0 and count >= N:
                    break
            elif log.action in ('do_deleteArticle',):
                deletedArticleList.append((log.forumId, log.articleId))

    def getRecentLogsForModifiedArticles(self, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        for log in self.recentLogGenerator(('do_editArticle', 'do_deleteArticle',)):
            if log.action in ('do_editArticle',):
                if (log.forumId, log.articleId) in deletedArticleList:
                    continue
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                count += 1
                if N != 0 and count >= N:
                    break
            elif log.action in ('do_deleteArticle',):
                deletedArticleList.append((log.forumId, log.articleId))

    def getRecentModifiedArticles(self, N=0, agoInSec=0):
        import bbslib
        groups = bbslib.getGroups()
        for log in self.getRecentLogsForModifiedArticles(N, agoInSec):
            yield groups.getArticle(log.forumId, log.threadId, log.articleId)

    def getRecentLogsForNewThreads(self, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        for log in self.recentLogGenerator(('do_newThread', 'do_deleteThread', 'do_moveThread')):
            if log.action in ('do_newThread',):
                if (log.forumId, log.threadId) in deletedArticleList:
                    continue
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                count += 1
                if N != 0 and count >= N:
                    break
            elif log.action == 'do_deleteThread':
                deletedArticleList.append((log.forumId, log.threadId))
            elif log.action == 'do_moveThread':
                for threadId in log.fromThreadIdList:
                    deletedArticleList.append((log.fromForumId, threadId))

    def getRecentNewThreads(self, N=0, agoInSec=0):
        import bbslib
        groups = bbslib.getGroups()
        for log in self.getRecentLogsForNewThreads(N):
            yield groups.getThread(log.forumId, log.threadId)

    def getRecentLogsForNewArticles(self, N=0, agoInSec=0):
        logs = list(self.getRecentLogsForNewThreads(N, agoInSec))
        logs.extend(list(self.getRecentLogsForReplies(N, agoInSec)))
        logs.sort(lambda a, b:cmp(b.time, a.time))
        if N:
            return logs[:N]
        return logs

    def getRecentNewArticles(self, N=0, agoInSec=0):
        import bbslib
        groups = bbslib.getGroups()
        for log in self.getRecentLogsForNewArticles(N, agoInSec):
            try:
                yield groups.getArticle(log.forumId, log.threadId, log.articleId)
            except:
                pass

    def getRecentNewArticlesFromDB(self, N=0, agoInSec=0):
        import dblib
        import bbslib
        if agoInSec:
            ticks = time.time() - agoInSec
            refDate = time.strftime('%Y/%m/%d %H:%M:%S', time.localtime(ticks))
            where = ' WHERE date > "%s"' % refDate
        else:
            where = ''
        orderBy = N and ' ORDER BY date DESC LIMIT %s' % N or ''
        cursor = dblib.getCursor()
        groups = bbslib.getGroups()
        idList = []
        for forumId in groups.getForumIdList():
            cursor.execute('SELECT date, id, threadId FROM %s%s%s' % (forumId, where, orderBy))
            for (date, articleId, threadId) in cursor.fetchall():
                idList.append((date, forumId, articleId, threadId))
        idList.sort()
        idList.reverse()
        if N:
            idList = idList[:N]
        for (date, forumId, articleId, threadId) in idList:
            yield groups.getArticle(forumId, threadId, articleId)

    def getUidListWhoReadAThread(self, forumId, threadId):
        #do_showThread
        uidList = []
        for log in self.recentLogGenerator(('do_showThread',)):
            if log.forumId == forumId and log.threadId == threadId:
                uidList.append((log.uid, log.time))
        return uidList

    def getRecentUidListWhoReadAThread(self, forumId, threadId):
        #do_showThread
        uidList = []
        uidDict = {}
        for log in self.recentLogGenerator(('do_showThread',)):
            if log.forumId == forumId and log.threadId == threadId and log.uid not in uidDict:
                uidList.append((log.uid, log.time))
                uidDict[log.uid] = None
        return uidList

    def getRecentLogsForEditedArticles(self, N=0, agoInSec=0):
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
        count = 0
        deletedArticleList = []
        processedList = []
        for log in self.recentLogGenerator(('do_editArticle', 'do_deleteArticle')):
            if log.action in ('do_editArticle',):
                if (log.forumId, log.threadId, log.articleId) in deletedArticleList:
                    continue
                if (log.forumId, log.threadId, log.articleId) in processedList:
                    continue
                if agoInSec != 0 and log.time < refStrTime:
                    break
                yield log
                processedList.append((log.forumId, log.threadId, log.articleId))
                count += 1
                if N != 0 and count >= N:
                    break
            elif log.action in ('do_deleteArticle', ):
                deletedArticleList.append((log.forumId, log.threadId, log.articleId))

    def getRecentEditedArticles(self, N=0, agoInSec=0):
        import bbslib
        groups = bbslib.getGroups()
        for log in self.getRecentLogsForEditedArticles(N, agoInSec):
            try:
                yield log, groups.getArticle(log.forumId, log.threadId, log.articleId)
            except:
                pass

    def getRecentEditedArticlesFromDB(self, N=0, agoInSec=0):
        import dblib
        import bbslib
        if agoInSec:
            ticks = time.time() - agoInSec
            refDate = time.strftime('%Y/%m/%d %H:%M:%S', time.localtime(ticks))
            where = ' WHERE mdate > "%s" and date != mdate' % refDate
        else:
            where = ''
        orderBy = N and ' ORDER BY mdate DESC LIMIT %s' % N or ''
        cursor = dblib.getCursor()
        groups = bbslib.getGroups()
        idList = []
        for forumId in groups.getForumIdList():
            cursor.execute('SELECT mdate, id, threadId FROM %s%s%s' % (forumId, where, orderBy))
            for (mdate, articleId, threadId) in cursor.fetchall():
                idList.append((mdate, forumId, articleId, threadId))
        idList.sort()
        idList.reverse()
        if N:
            idList = idList[:N]
        for (mdate, forumId, articleId, threadId) in idList:
            yield groups.getArticle(forumId, threadId, articleId)

def refreshWriteLogFile(since=0, agoInSec=0):
    if since:
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(since))
    else:
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
    web = WebLogAnalyzer()
    fw = open(WRITE_LOG_FILE, 'w')
    for log in web.logGenerator(('do_newThread', 'do_reply', 
            'do_deleteArticle', 'do_editArticle',
            'do_deleteThread', 'do_moveThread')):
        if agoInSec and log.time < refStrTime:
            continue
        fw.write('%s\n' % repr(log))

def packWriteLogFile(since=0, agoInSec=0):
    if since:
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(since))
    else:
        refStrTime = time.strftime('%Y/%m/%d:%H:%M:%S', time.localtime(time.time() - agoInSec))
    web = WebLogAnalyzer(WRITE_LOG_FILE)
    logList = list(web.logGenerator('*'))

    fw = open(os.path.join(config.systemFolder, WRITE_LOG_FILE), 'w')
    for log in logList:
        if agoInSec and log.time < refStrTime:
            continue
        fw.write('%s\n' % repr(log))
