'''
    PyBB - Template Processor

    Copyright (c) 2003-2004 Gang Seong Lee <gslee@mail.kw.ac.kr>
    All rights reserved, see COPYING for details.
'''

import re
import sys, os
import StringIO
import stat
#import cPickle
import marshal
import cStringIO
import config

TEXT, STATEMENT, EXPRESSION, INCLUDE = range(4)

splitPattern = re.compile('<:(.*?):>', re.DOTALL)
def splitTeul(text):
    start = 0
    m = splitPattern.search(text)
    L = []
    while m:
        s, e = m.span()
        s += start
        e += start
        html = text[start:s]
        code = m.group(1)
        if html:
            L.append((TEXT, html))
        if code.startswith('#statements'):
            L.append((STATEMENT, code[len('#statements'):].lstrip()))
        elif code.startswith('#include'):
            L.extend(splitTeul(file(os.path.join(config.skinPath, code.split()[1]), 'r').read()))
        else:
            L.append((EXPRESSION, code))
        start = e
        m = splitPattern.search(text[start:])
    if text[start:]:
        L.append((TEXT, text[start:]))
    return L

def codeCompile(args):
    ctype, text = args
    if ctype in [TEXT, INCLUDE]:
        return args
    if ctype == EXPRESSION:
        return ctype, compile(text.lstrip(), '<string>', 'eval')
    if ctype == STATEMENT:
        return ctype, compile(text, '<string>', 'exec')

def putItTogether(blocks):
    L = []
    for ctype, text in blocks:
        if ctype == TEXT:
            L.append('sys.stdout.write("""%s""")' % text.replace('"', '\\"'))
        elif ctype == EXPRESSION:
            L.append('sys.stdout.write(str(%s))' % text)
        elif ctype == STATEMENT:
            L.append(text)
    return '\n'.join(L)
        
def makeTeulc(filePath):
    teul = file(filePath).read()
    blocks = splitTeul(teul)
    pythonCode = putItTogether(blocks)
    compiledCode = compile(pythonCode, '<string>', 'exec')
    #cPickle.dump(compiledCodeList, file(os.path.splitext(filePath)[0]+'.teulc', 'w'))
    marshal.dump(compiledCode, file(os.path.splitext(filePath)[0]+'.teulc', 'wb'))
    return compiledCode

def execCode(code, globalNS={}, localNS={}):
    try:
        localNS['sys'] = sys
        stdout = sys.stdout
        buf = StringIO.StringIO()
        sys.stdout = buf
        exec code in globalNS, localNS
        sys.stdout = stdout
        return buf.getvalue()
#        if ctype == INCLUDE:     # FIXME
#            path = os.path.join(config.skinPath, code)
#            compiledCodeList = getTemplate(path)
#            L = []
#            for compiledCode in compiledCodeList:
#                L.append(execCode(compiledCode, globalNS, localNS))
#            return ''.join(map(str, L))
        #if ctype == INCLUDE:
        #    return getTemplate(code)
    except Exception, errMsg:
        sys.stdout = stdout
        return 'ERROR:%s' % (str(errMsg))
            

def checkTeulIsModified(filePath, teulcPath):
    return False
    mtime = os.stat(filePath)[stat.ST_MTIME]
    mtimeOfTeulc = os.stat(teulcPath)[stat.ST_MTIME]
    if mtime >= mtimeOfTeulc:
        return True
    return False

def joinTextBlock(block1, block2):
    ctype1, text1 = block1
    ctype2, text2 = block2
    if ctype1 == ctype2 and ctype1 == TEXT:
        return (ctype1, text1+text2) 
    return block1, block2
        

templateTag = re.compile('<:(.*?(<:.*?:>.*?)*):>', re.DOTALL)
def templateProcessor(template, globalNS=None, localNS=None):
    if globalNS == None:
        globalNS = {}
    if localNS == None:
        localNS = {}
    def repl(match):
        try:
            stdout = sys.stdout
            block = match.group(1).strip()
            if not block:
                return ''
            if block[0] != '#':
                return templateProcessor(str(eval(block, globalNS, localNS)), globalNS, localNS)
            lines = block.split('\n')
            line = lines[0]
            if line.strip() == '#statements':
                sbuf = StringIO.StringIO()
                codeBlock = '\n'.join(lines[1:]) + '\n'
                sys.stdout = sbuf
                exec codeBlock in globalNS, localNS
                sys.stdout = stdout
                return templateProcessor(sbuf.getvalue(), globalNS, localNS)
            elif line.startswith('#include'):
                fname = line.split()[1]
                path = os.path.join(config.skinPath, fname)
                return templateProcessor(getTemplate(path), globalNS, localNS)
        except Exception, errMsg:
            sys.stdout = stdout
            return 'ERROR:%s (%s)' % (str(errMsg), block)
    if config.debugMode or type(template) == type(''):
        html = templateTag.sub(repl, template)
    else:
        html = execCode(template, globalNS, localNS)
    return html

#tempCache = {}
def getTemplate(path, aaa=None):
    path = os.path.normpath(path)
    if config.debugMode:
        if aaa:
            file('adsfasdfasdf', 'w').write('1')
        template = open(path).read()
        return template
#    template = tempCache.get(path)
    #if template:
    #    return template
    teulcPath = os.path.splitext(path)[0] + '.teulc'
    if os.path.exists(teulcPath) and not checkTeulIsModified(path, teulcPath):
        compiledCodeList = marshal.load(file(teulcPath, 'rb'))
    else:
        compiledCodeList = makeTeulc(path)
#    tempCache[path] = compiledCodeList 
    return compiledCodeList
